#include <tulip/ExportModule.h>
#include <tulip/Graph.h>
#include <tulip/ConnectedTest.h>
#include <tulip/StringCollection.h>
#include <tulip/LayoutProperty.h>

#include <portgraph/PorgyTlpGraphStructure.h>
#include <portgraph/PortGraphModel.h>

using namespace tlp;
using namespace std;

namespace {
static const char * paramHelp[] = {
  // name
  HTML_HELP_OPEN() \
  HTML_HELP_DEF( "type", "string" ) \
  HTML_HELP_DEF( "default", "" ) \
  HTML_HELP_BODY() \
  "Name of the graph being exported." \
  HTML_HELP_CLOSE(),
  // author
  HTML_HELP_OPEN() \
  HTML_HELP_DEF( "type", "string" ) \
  HTML_HELP_DEF( "default", "" ) \
  HTML_HELP_BODY() \
  "Authors" \
  HTML_HELP_CLOSE(),
  // comments
  HTML_HELP_OPEN() \
  HTML_HELP_DEF( "type", "string" ) \
  HTML_HELP_DEF( "default", "This file was generated by Tulip." ) \
  HTML_HELP_BODY() \
  "Description of the graph." \
  HTML_HELP_CLOSE(),
};
}

class PorgyRuleExport : public ExportModule {
private:
    std::map<unsigned int, unsigned int> nodeMap;
    std::map<unsigned int, unsigned int> edgeMap;

    void buildNodeMap(Graph *sourceGraph){
        tlp::node currentNode;
        unsigned int newNode_id=0;
        forEach(currentNode, sourceGraph->getNodes()){
            nodeMap.insert(std::pair<unsigned int, unsigned int>(currentNode.id, newNode_id));
            newNode_id++;
        }
    }

    void buildEdgeMap(Graph *sourceGraph){
        tlp::edge currentEdge;
        unsigned int newEdge_id = 0;
        forEach(currentEdge, sourceGraph->getEdges()){
            edgeMap.insert(std::pair<unsigned int, unsigned int>(currentEdge.id, newEdge_id));
            newEdge_id++;
        }
    }

    void transformAlgorithm(Graph *sourceGraph) {
        std::string algorithm;
        sourceGraph->getAttribute<std::string>("Rule Algorithm", algorithm);
        std::string ancient, recent, strIdentifier="#?*";
        std::size_t index;
        for (auto& it : nodeMap) {
            stringstream ss1;
            ss1 << it.first;
            ancient = "node("+ ss1.str() +")";
            stringstream ss2;
            ss2 << it.second;
            recent = "node("+strIdentifier+ ss2.str() +")";
            index = algorithm.find(ancient);
            while(index!=std::string::npos) {
                algorithm.replace(index, ancient.length(), recent);
                index = algorithm.find(ancient);
            }
            ancient = "n("+ ss1.str() +")";
            recent = "n("+strIdentifier+ ss2.str() +")";
            index = algorithm.find(ancient);
            while(index!=std::string::npos) {
                algorithm.replace(index, ancient.length(), recent);
                index = algorithm.find(ancient);
            }
        }
        for (auto& it : edgeMap) {
            stringstream ss1;
            ss1 << it.first;
            ancient = "edge("+ ss1.str() +")";
            stringstream ss2;
            ss2 << it.second;
            recent = "edge("+strIdentifier+ ss2.str() +")";
            index = algorithm.find(ancient);
            while(index!=std::string::npos) {
                algorithm.replace(index, ancient.length(),recent);
                index = algorithm.find(ancient);
            }
            ancient = "e("+ ss1.str() +")";
            recent = "e("+strIdentifier+ ss2.str() +")";
            index = algorithm.find(ancient);
            while(index!=std::string::npos) {
                algorithm.replace(index, ancient.length(), recent);
                index = algorithm.find(ancient);
            }
        }
        ancient = "("+strIdentifier;
        recent = "(";
        index = algorithm.find(ancient);
        while(index!=std::string::npos) {
            algorithm.replace(index, ancient.length(), recent);
            index = algorithm.find(ancient);
        }
        sourceGraph->setAttribute<std::string>("Rule Algorithm", algorithm);
    }

public:
    PLUGININFORMATION("Rule Export","Name","Date","Description","1.1",PorgyConstants::CATEGORY_NAME)
    PorgyRuleExport(tlp::PluginContext* context):ExportModule(context) {
        addInParameter<string>("name", paramHelp[0],"");
        addInParameter<string>("author", paramHelp[1],"");
        addInParameter<string>("text::comments", paramHelp[2], "This file was generated by Tulip.");

        addDependency("TLP Export", "1.0");
    }

    string fileExtension() const {
      return "porgy.rule";
    }

    bool exportGraph(ostream &os) {
        if(graph!=nullptr) {
            Observable::holdObservers();

            buildNodeMap(graph);
            buildEdgeMap(graph);

            std::string algorithm;
            graph->getAttribute<std::string>("Rule Algorithm", algorithm);
            transformAlgorithm(graph);

            bool result = tlp::exportGraph(graph, os, "TLP Export", *dataSet, pluginProgress);

            graph->setAttribute<std::string>("Rule Algorithm", algorithm);

            Observable::unholdObservers();
            return result;
        }
        else {
            std::cerr << "Graph is empty" << endl;
            return false;
        }
    }
};

PLUGIN(PorgyRuleExport)
