#include "UtreeWalk.h"
#include "RunStrategyElement.h"

#include <tulip/ForEach.h>
#include <tulip/BooleanProperty.h>

#include <portgraph/PortGraphModel.h>

#include <boost/lexical_cast.hpp>

#include <regex>

using namespace tlp;
using namespace std;
using namespace boost;

//execute the utree structure to compute the booleanproperty
utree_walk::utree_walk(vector<booleanptr> &vect, string &p, string &ban, Graph *g) : vect(vect), _p(p), _ban(ban), g(g) {}

void utree_walk::operator()(spirit::utree::invalid_type) const {
    assert(false);
}

void utree_walk::operator()(spirit::utree::nil_type) const {
    assert(false);
}

void utree_walk::operator()(bool) const {
    assert(false);
}

//used for crtban, crtpos, crtgraph, emptyset
void utree_walk::operator()(spirit::binary_range_type const& b) const {
    std::string st="";
    for (spirit::binary_range_type::const_iterator i = b.begin(); i != b.end(); ++i)
        st += i;

    StrategyElement::focus_operation op = static_cast<StrategyElement::focus_operation>(stoul(st));
    switch(op) {
    case StrategyElement::focus_operation::CRTBAN:
    {
        booleanptr banp(new BooleanProperty(g));
        if(!_ban.empty())
            banp->copy(g->getProperty<BooleanProperty>(_ban));
        vect.push_back(banp);
    }
        break;
    case StrategyElement::focus_operation::CRTPOS:
    {
        booleanptr pp(new BooleanProperty(g));
        pp->copy(g->getProperty<BooleanProperty>(_p));
        vect.push_back(pp);
    }
        break;
    case StrategyElement::focus_operation::CRTGRAPH:
    {
        //cannot use setAllNodeValue because it changes the default value
        booleanptr ptr(new BooleanProperty(g));
        node n;
        forEach(n, g->getNodes()) {
            ptr->setNodeValue(n, true);
        }
        edge e;
        forEach(e, g->getEdges()) {
            ptr->setEdgeValue(e, true);
        }
        vect.push_back(ptr);
    }
        break;
    case StrategyElement::focus_operation::EMPTYSET:
    {
        //cannot use setAllNodeValue because it changes the default value
        booleanptr ptr(new BooleanProperty(g));
        node n;
        forEach(n, g->getNodes()) {
            ptr->setNodeValue(n, false);
        }
        edge e;
        forEach(e, g->getEdges()) {
            ptr->setEdgeValue(e, false);
        }
        vect.push_back(ptr);
    }
        break;
    }
}

//one(F) and all(F)
void utree_walk::operator()(spirit::utf8_string_range_type const& type) const {
    string st="";
    for (spirit::utf8_string_range_type::const_iterator i = type.begin(); i != type.end(); ++i)
        st += *i;
    if(st=="one") {
        booleanptr p1(vect.back());
        vect.pop_back();
        booleanptr p2(new BooleanProperty(g));
        node n;
        vector<node> v;
        forEach(n, p1->getNodesEqualTo(true)) {
            v.push_back(n);
        }
        uniform_int_distribution<int> choose(0, v.size()-1);
        n = v[choose(PorgyTlpGraphStructure::gen)]; //chose one element
        PortGraphModel pg(g);  //one returns a graph, so select the whole portnode
        PortNode *pn(pg[n]);
        pn->select(true, p2.get());
        vect.push_back(p2);
    }
//#ifndef NDEBUG
//    else if(st=="all") {
//        cerr << __PRETTY_FUNCTION__ << ": nothing to do for all(t)" << endl;
//    }
//#endif

}

void utree_walk::operator()(spirit::utf8_symbol_range_type const& str) const {
    typedef spirit::utf8_symbol_range_type::const_iterator iterator;
    string st="";
    for (iterator i = str.begin(); i != str.end(); ++i)
        st += *i;
    booleanptr p1(vect.back());
    vect.pop_back();
    booleanptr p2(vect.back());
    vect.pop_back();
    booleanptr p3(new BooleanProperty(g));
    p3->copy(p2.get());
    node n;
    edge e;
    StrategyElement::set_theory_op op=static_cast< StrategyElement::set_theory_op>(lexical_cast<unsigned>(st));
    switch(op) {
    case StrategyElement::set_theory_op::MINUS:
        //A \ B : keeps all elements of A set to true which are not true in B
        forEach(n, p3->getNodesEqualTo(true)) {
            if(p1->getNodeValue(n)) {
                p3->setNodeValue(n, false);
            }
        }
        forEach(e, p3->getEdgesEqualTo(true)) {
            if(p2->getEdgeValue(e))
                p3->setEdgeValue(e, false);
        }
        break;
    case StrategyElement::set_theory_op::UNION:
        // A \cup B = all elements set to true in A or B
        forEach(n, p1->getNodesEqualTo(true)) {
            p3->setNodeValue(n, true);
        }
        forEach(e, p1->getEdgesEqualTo(true)) {
            p3->setEdgeValue(e, true);
        }
        break;
    case StrategyElement::set_theory_op::INTER:
        //A \cap B = all elements set to true in A and B
        forEach(n, p3->getNodesEqualTo(true)) {
            if(!p1->getNodeValue(n))
                p3->setNodeValue(n, false);
        }
        forEach(e, p3->getEdgesEqualTo(true)) {
            if(!p1->getEdgeValue(e))
                p3->setEdgeValue(e, false);
        }
        break;
    }
    vect.push_back(p3);
}

/////////////////////////
static void propertyOp(node &c, PortNode *pn, booleanptr check, BooleanProperty *prop_new, PropertyInterface *prop, StrategyElement::relational_operators &op, string& value) {
    switch(op) {
    case StrategyElement::relational_operators::EQ:
    {
        if((check->getNodeValue(c))&&(prop->getNodeStringValue(c)==value)) {
            pn->select(true, prop_new);
        }
        break;
    }
    case StrategyElement::relational_operators::NEQ:
    {
        if((check->getNodeValue(c))&&(prop->getNodeStringValue(c)!=value)) {
            pn->select(true, prop_new);
        }
        break;
    }
    case StrategyElement::relational_operators::GT:
    {
        if((check->getNodeValue(c))&&(prop->getNodeStringValue(c)>value)) {
            pn->select(true, prop_new);
        }
        break;
    }
    case StrategyElement::relational_operators::GTE:
    {
        if((check->getNodeValue(c))&&(prop->getNodeStringValue(c)>=value)) {
            pn->select(true, prop_new);
        }
        break;
    }
    case StrategyElement::relational_operators::LTE:
    {
        if((check->getNodeValue(c))&&(prop->getNodeStringValue(c)<=value)) {
            pn->select(true, prop_new);
        }
        break;
    }
    case StrategyElement::relational_operators::LT:
    {
        if((check->getNodeValue(c))&&(prop->getNodeStringValue(c)<value)) {
            pn->select(true, prop_new);
        }
        break;
    }
      case StrategyElement::relational_operators::REGEXP:
      {
          regex val_regex(value);
          if((check->getNodeValue(c))&&(regex_match(prop->getNodeStringValue(c), val_regex))) {
              pn->select(true, prop_new);
          }
          break;
      }
    }
}

void select_edge(booleanptr fromOp, const pair<node, node>&ends, PortGraphModel& pg, BooleanProperty *prop_new) {
    if(fromOp->getNodeValue(ends.first)&&(!fromOp->getNodeValue(ends.second))) {
        PortNode *pn(pg[ends.second]);
        pn->select(true, prop_new);
        return;
    }
    if(fromOp->getNodeValue(ends.second)&&(!fromOp->getNodeValue(ends.first))) {
        PortNode *pn(pg[ends.first]);
        pn->select(true, prop_new);
        return;
    }
}

//visitor for the last parameter of the property construct
class ValueVisitor : public static_visitor<string> {
    PropertyInterface *_p;
    Graph* g;
public:
    ValueVisitor(PropertyInterface *p, Graph *g):_p(p),g(g){}
    string operator()(const TulipProp &prop) {
         _p = g->getProperty(prop.name);
         return "";
    }
    string operator()(const std::string &str) {
        return str;
    }
};

class expr_visitor : public static_visitor<bool> {

public:
   bool operator()(const string& ) {
       return false;
   }
   bool operator()(const StrategyElement::ngbprop_ds& ) {
       return true;
   }
};

void utree_walk::operator()(spirit::any_ptr const&a) const{
    //right now, I am using any_ptr only for StrategyElement::property_datastruct
    StrategyElement::property_datastruct *v_ptr = a.get<StrategyElement::property_datastruct*>();
    StrategyElement::property_datastruct v(*v_ptr);
     //do not delete in case of a while. I have a memory leak here.
    //  delete v_ptr;

    //starting set
    spirit::utree::visit(get<1>(v), *this);
    booleanptr fromOp(vect.back());
    vect.pop_back();
    PortGraphModel pg(g);
    StrategyElement::ngbprop_ds expr;
    booleanptr prop_new_ptr(new BooleanProperty(g));
    expr_visitor ex;
    if(get<3>(v).apply_visitor(ex)) {
        expr = boost::get<StrategyElement::ngbprop_ds>(get<3>(v));
        //what to find
        TulipProp check_property(get<0>(expr));
        StrategyElement::relational_operators op(get<1>(expr));
        PropertyInterface *prop = g->getProperty(check_property.name);
        PropertyInterface *p2=nullptr;
        ValueVisitor visit(p2, g);
        string value = get<2>(expr).apply_visitor(visit);
        //property or Ngb
        if(get<0>(v)==StrategyElement::position_keywords::NGB) {
            PortNode* pn;
            BooleanProperty *prop_new = prop_new_ptr.get();
            switch(get<2>(v)) {
            case StrategyElement::Graph_Element::NODE:
            {
                forEach(pn, pg.getPortNodes()) {
                    const  PortNode *pn2=nullptr;
                    string val = (value.empty())?p2->getNodeStringValue(*pn):value;
                    node c(pn->getCenter());
                    switch(op) {
                    case StrategyElement::relational_operators::EQ:
                    {
                        if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)==val)) {
                            forEach(pn2,pn->getInOutConnectedPortNodes() ) {
                                if(!fromOp->getNodeValue(pn2->getCenter()))
                                    pn2->select(true, prop_new);
                            }
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::NEQ:
                    {
                        if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)!=val)) {
                            forEach(pn2, pn->getInOutConnectedPortNodes()) {
                                if(!fromOp->getNodeValue(pn2->getCenter()))
                                    pn2->select(true, prop_new);
                            }
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::GT:
                    {
                        if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)>val)) {
                            forEach(pn2, pn->getInOutConnectedPortNodes()) {
                                if(!fromOp->getNodeValue(pn2->getCenter()))
                                    pn2->select(true, prop_new);
                            }
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::GTE:
                    {
                        if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)>=val)) {
                            forEach(pn2, pn->getInOutConnectedPortNodes()) {
                                if(!fromOp->getNodeValue(pn2->getCenter()))
                                    pn2->select(true, prop_new);
                            }
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::LTE:
                    {
                        if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)<=val)) {
                            forEach(pn2, pn->getInOutConnectedPortNodes()) {
                                if(!fromOp->getNodeValue(pn2->getCenter()))
                                    pn2->select(true, prop_new);
                            }
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::LT:
                    {
                        if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)<val)) {
                            forEach(pn2, pn->getInOutConnectedPortNodes()) {
                                if(!fromOp->getNodeValue(pn2->getCenter()))
                                    pn2->select(true, prop_new);
                            }
                        }
                        break;
                    }
                      case StrategyElement::relational_operators::REGEXP:
                        {
                          regex val_regex(val);
                          if((fromOp->getNodeValue(c))&&(regex_match(prop->getNodeStringValue(c), val_regex))) {
                              forEach(pn2, pn->getInOutConnectedPortNodes()) {
                                  if(!fromOp->getNodeValue(pn2->getCenter()))
                                      pn2->select(true, prop_new);
                              }
                          }
                          break;
                        }
                    }
                }
            }
                break;
            case StrategyElement::Graph_Element::PORT:
            {
                forEach(pn, pg.getPortNodes()) {
                    Port::PortPtr pt;
                    forEach(pt, pn->getPorts()) {
                        string val = (value.empty())?p2->getNodeStringValue(*pt.get()):value;
                        Port::PortPtr p;
                        node c(*pt.get());
                        switch(op) {
                        case StrategyElement::relational_operators::EQ:
                        {
                            if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)==val)) {
                                forEach(p, pt->getConnectedPorts()) {
                                    if(!fromOp->getNodeValue(p->getParentPortNode()->getCenter()))
                                        p->getParentPortNode()->select(true, prop_new);
                                }
                            }
                            break;
                        }
                        case StrategyElement::relational_operators::NEQ:
                        {
                            if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)!=val)) {
                                forEach(p, pt->getConnectedPorts()) {
                                    if(!fromOp->getNodeValue(p->getParentPortNode()->getCenter()))
                                        p->getParentPortNode()->select(true, prop_new);
                                }
                            }
                            break;
                        }
                        case StrategyElement::relational_operators::GT:
                        {
                            if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)>val)) {
                                forEach(p, pt->getConnectedPorts()) {
                                    if(!fromOp->getNodeValue(p->getParentPortNode()->getCenter()))
                                        p->getParentPortNode()->select(true, prop_new);
                                }
                            }
                            break;
                        }
                        case StrategyElement::relational_operators::GTE:
                        {
                            if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)>=val)) {
                                forEach(p, pt->getConnectedPorts()) {
                                    if(!fromOp->getNodeValue(p->getParentPortNode()->getCenter()))
                                        p->getParentPortNode()->select(true, prop_new);
                                }
                            }
                            break;
                        }
                        case StrategyElement::relational_operators::LTE:
                        {
                            if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)<=val)) {
                                forEach(p, pt->getConnectedPorts()) {
                                    if(!fromOp->getNodeValue(p->getParentPortNode()->getCenter()))
                                        p->getParentPortNode()->select(true, prop_new);
                                }
                            }
                            break;
                        }
                        case StrategyElement::relational_operators::LT:
                        {
                            if((fromOp->getNodeValue(c))&&(prop->getNodeStringValue(c)<val)) {
                                forEach(p, pt->getConnectedPorts()) {
                                    if(!fromOp->getNodeValue(p->getParentPortNode()->getCenter()))
                                        p->getParentPortNode()->select(true, prop_new);
                                }
                            }
                            break;
                        }
                          case StrategyElement::relational_operators::REGEXP:
                          {
                              regex val_regex(val);
                              if((fromOp->getNodeValue(c))&&(regex_match(prop->getNodeStringValue(c), val_regex))) {
                                  forEach(p, pt->getConnectedPorts()) {
                                      if(!fromOp->getNodeValue(p->getParentPortNode()->getCenter()))
                                          p->getParentPortNode()->select(true, prop_new);
                                  }
                              }
                              break;
                          }
                        }
                    }
                }
            }
                break;
            case StrategyElement::Graph_Element::EDGE:
            {
                edge e;
                forEach(e, pg.getEdges()) {
                    const pair<node, node>&ends = g->ends(e);
                    string val = (value.empty())?p2->getEdgeStringValue(e):value;
                    switch(op) {
                    case StrategyElement::relational_operators::EQ:
                    {
                        if(prop->getEdgeStringValue(e)==val)
                            select_edge(fromOp, ends, pg, prop_new);
                    }
                        break;
                    case StrategyElement::relational_operators::NEQ:
                    {
                        if(prop->getEdgeStringValue(e)!=val)
                            select_edge(fromOp, ends, pg, prop_new);
                    }
                        break;
                    case StrategyElement::relational_operators::LT:
                    {
                        if(prop->getEdgeStringValue(e)<val)
                            select_edge(fromOp, ends, pg, prop_new);
                    }
                        break;
                    case StrategyElement::relational_operators::LTE:
                    {
                        if(prop->getEdgeStringValue(e)<=val)
                            select_edge(fromOp, ends, pg, prop_new);
                    }
                        break;
                    case StrategyElement::relational_operators::GT:
                    {
                        if(prop->getEdgeStringValue(e)>val)
                            select_edge(fromOp, ends, pg, prop_new);
                    }
                        break;
                    case StrategyElement::relational_operators::GTE:
                    {
                        if(prop->getEdgeStringValue(e)>=val)
                            select_edge(fromOp, ends, pg, prop_new);
                    }
                        break;
                      case StrategyElement::relational_operators::REGEXP:
                        {
                          regex val_regex(val);
                          if(regex_match(prop->getEdgeStringValue(e), val_regex))
                            select_edge(fromOp, ends, pg, prop_new);
                        }
                        break;
                    }
                }
            }
                break;
            }
        }
        else { //property operator used
            switch(get<2>(v)) {
            case StrategyElement::Graph_Element::NODE:
            {
                PortNode* pn;
                forEach(pn, pg.getPortNodes()) {
                    node n = pn->getCenter();
                    string val = (value.empty())?p2->getNodeStringValue(n):value;
                    propertyOp(n, pn, fromOp, prop_new_ptr.get(), prop, op, val);
                }
            }
                break;
            case StrategyElement::Graph_Element::PORT:
                PortNode* pn;
                forEach(pn, pg.getPortNodes()) {
                    Port::PortPtr pt;
                    forEach(pt, pn->getPorts()) {
                        node n = pt->getNode();
                        string val = (value.empty())?p2->getNodeStringValue(n):value;
                        propertyOp(n, pn, fromOp, prop_new_ptr.get(), prop, op, val);
                    }
                }
                break;
            case StrategyElement::Graph_Element::EDGE:
            {
                edge e;
                BooleanProperty *prop_new = prop_new_ptr.get();
                forEach(e, pg.getEdges()) {
                    if(value.empty())
                        value = p2->getEdgeStringValue(e);
                    switch(op) {
                    case StrategyElement::relational_operators::EQ:
                    {
                        if((fromOp->getEdgeValue(e))&&(prop->getEdgeStringValue(e)==value)) {
                            prop_new->setEdgeValue(e, true);
                            const pair<node, node> &ends = g->ends(e);
                            pg[ends.first]->select(true, prop_new);
                            pg[ends.second]->select(true, prop_new);
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::NEQ:
                    {
                        if((fromOp->getEdgeValue(e))&&(prop->getEdgeStringValue(e)!=value)) {
                            prop_new->setEdgeValue(e, true);
                            const pair<node, node> &ends = g->ends(e);
                            pg[ends.first]->select(true, prop_new);
                            pg[ends.second]->select(true, prop_new);
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::GT:
                    {
                        if((fromOp->getEdgeValue(e))&&(prop->getEdgeStringValue(e)>value)) {
                            prop_new->setEdgeValue(e, true);
                            const pair<node, node> &ends = g->ends(e);
                            pg[ends.first]->select(true, prop_new);
                            pg[ends.second]->select(true, prop_new);
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::GTE:
                    {
                        if((fromOp->getEdgeValue(e))&&(prop->getEdgeStringValue(e)>=value)) {
                            prop_new->setEdgeValue(e, true);
                            const pair<node, node> &ends = g->ends(e);
                            pg[ends.first]->select(true, prop_new);
                            pg[ends.second]->select(true, prop_new);
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::LTE:
                    {
                        if((fromOp->getEdgeValue(e))&&(prop->getEdgeStringValue(e)<=value)) {
                            prop_new->setEdgeValue(e, true);
                            const pair<node, node> &ends = g->ends(e);
                            pg[ends.first]->select(true, prop_new);
                            pg[ends.second]->select(true, prop_new);
                        }
                        break;
                    }
                    case StrategyElement::relational_operators::LT:
                    {
                        if((fromOp->getEdgeValue(e))&&(prop->getEdgeStringValue(e)<value)) {
                            prop_new->setEdgeValue(e, true);
                            const pair<node, node> &ends = g->ends(e);
                            pg[ends.first]->select(true, prop_new);
                            pg[ends.second]->select(true, prop_new);
                        }
                        break;
                    }
                      case StrategyElement::relational_operators::REGEXP:
                      {
                          regex value_regex(value);
                          if((fromOp->getEdgeValue(e))&&(regex_match(prop->getEdgeStringValue(e), value_regex))) {
                              prop_new->setEdgeValue(e, true);
                              const pair<node, node> &ends = g->ends(e);
                              pg[ends.first]->select(true, prop_new);
                              pg[ends.second]->select(true, prop_new);
                          }
                          break;
                      }
                    }
                }
            }
            }
        }
    }
    else { //True was used as Expr.
         BooleanProperty *prop_new = prop_new_ptr.get();
         PortNode *pn;
         forEach(pn, pg.getPortNodes()) {
             const  PortNode *pn2=nullptr;
             if(fromOp->getNodeValue(pn->getCenter())) {
                 forEach(pn2,pn->getInOutConnectedPortNodes() ) {
                     if(!fromOp->getNodeValue(pn2->getCenter()))
                         pn2->select(true, prop_new);
                 }
             }
         }
    }
    vect.push_back(prop_new_ptr);
}

void utree_walk::operator()(spirit::function_base const&) const {
    assert(false);
}

#ifndef NDEBUG
//visitor for the last parameter of the property construct
class ValueVisitorPrint : public static_visitor<> {
    ostream &out;
public:
    ValueVisitorPrint(ostream &out):out(out){}
    void operator()(const TulipProp &prop) {
        out << "Prop:" << prop.name;
    }
    void operator()(const std::string &str) {
        out << "Value:" << str;
    }
};

utree_print::utree_print(std::ostream& out) : out(out) {}

void utree_print::operator()(spirit::utree::invalid_type) const {
    out << "<invalid> ";
}

void utree_print::operator()(spirit::utree::nil_type) const {
    out << "<nil> ";
}

void utree_print::operator()(bool b) const {
    out << (b ? "true" : "false") << ' ';
}

//used for crtban, crtpos, crtgraph, emptyset
void utree_print::operator()(spirit::binary_range_type const& b) const {
    typedef spirit::binary_range_type::const_iterator iterator;
    std::string st="";
    for (iterator i = b.begin(); i != b.end(); ++i)
        st += i;

    StrategyElement::focus_operation op = static_cast<StrategyElement::focus_operation>(stoul(st));
    switch(op) {
    case StrategyElement::focus_operation::CRTBAN:
        out << " CRTBAN ";
        break;
    case StrategyElement::focus_operation::CRTPOS:
        out << " CRTPOS ";
        break;
    case StrategyElement::focus_operation::CRTGRAPH:
        out << " CRTGRAPH ";
        break;
    case StrategyElement::focus_operation::EMPTYSET:
        out << " EMPTYSET ";
        break;
    }
}

void utree_print::operator()(spirit::utf8_string_range_type const& str) const {
    typedef spirit::utf8_string_range_type::const_iterator iterator;
    out << '"';
    for (iterator i = str.begin(); i != str.end(); ++i)
        out << *i;
    out << "\" ";
}

void utree_print::operator()(spirit::utf8_symbol_range_type const& str) const {
    out << "op(";
    typedef spirit::utf8_symbol_range_type::const_iterator iterator;
    string st="";
    for (iterator i = str.begin(); i != str.end(); ++i)
        st += *i;
    StrategyElement::set_theory_op op=static_cast<StrategyElement::set_theory_op>(stoul(st));
    switch(op) {
    case StrategyElement::set_theory_op::MINUS:
        out << "MINUS";
        break;
    case StrategyElement::set_theory_op::UNION:
        out << "UNION";
        break;
    case StrategyElement::set_theory_op::INTER:
        out << "INTER";
        break;
    }

    out << ") ";
}

void utree_print::operator()(spirit::any_ptr const&a) const {
    //right now, I am using any_ptr only for StrategyElement::property_datastruct => very ugly
    StrategyElement::property_datastruct *v_ptr = a.get<StrategyElement::property_datastruct*>();
    StrategyElement::property_datastruct v(*v_ptr);
    //delete v_ptr; => memory leak but I do not know how to solve it. the pointer cannot be deleted if the strategy uses a loop operator
    //operation
    switch(get<0>(v)) {
    case StrategyElement::position_keywords::PROPERTY:
        out << "Property(";
        break;
    case StrategyElement::position_keywords::NGB:
        out << "Ngb(";
        break;
    }
    //starting set
    spirit::utree::visit(get<1>(v), *this);
    out << ", ";

    //graph element
    switch(get<2>(v)) {
    case StrategyElement::Graph_Element::NODE:
        out << "Node";
        break;
    case StrategyElement::Graph_Element::EDGE:
        out << "Edge";
        break;
    case StrategyElement::Graph_Element::PORT:
        out << "Port";
        break;
    }

    out << ", ";
    //operation
    expr_visitor ex;
    if(get<3>(v).apply_visitor(ex)) {
        StrategyElement::ngbprop_ds p_ex(boost::get<StrategyElement::ngbprop_ds>(get<3>(v)));
        TulipProp prop(get<0>(p_ex));
        out << prop.name << " ";
        switch(get<1>(p_ex)) {
        case StrategyElement::relational_operators::EQ:
            out << "EQ";
            break;
        case StrategyElement::relational_operators::NEQ:
            out << "NEQ";
            break;
        case StrategyElement::relational_operators::GT:
            out << "GT";
            break;
        case StrategyElement::relational_operators::GTE:
            out << "GTE";
            break;
        case StrategyElement::relational_operators::LT:
            out << "LT";
            break;
        case StrategyElement::relational_operators::LTE:
            out << "LTE";
            break;
          case StrategyElement::relational_operators::REGEXP:
            out << "REGEXP";
            break;
        }

        out << " ";
        ValueVisitorPrint visit(out);
        get<2>(p_ex).apply_visitor(visit);
    }
    else {
        out << "True";
    }

    out << ") ";

}

void utree_print::operator()(spirit::function_base const&) const {
    return (*this)("<function>");
}

std::ostream& operator<<(std::ostream& out, spirit::utree const& x)
{
    spirit::utree::visit(x, utree_print(out));
    return out;
}
#endif
