/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "InteractorPortGraphAddEdgeComponent.h"

#include <QMouseEvent>

#include <tulip/View.h>
#include <tulip/StringProperty.h>
#include <tulip/GlMainWidget.h>
#include <tulip/GlGraphComposite.h>
#include <tulip/Observable.h>

#include <portgraph/PorgyTlpGraphStructure.h>
#include <portgraph/PortGraphRule.h>
#include <portgraph/Bridge.h>

using namespace tlp;
using namespace std;

PortGraphAddEdgeComponent::PortGraphAddEdgeComponent() {

}

void PortGraphAddEdgeComponent::addLink(QObject *widget, const tlp::node src, const tlp::node target) {
    Graph * g = view()->graph();
    if(!PorgyTlpGraphStructure::isRuleGraph(g)) {
        MouseEdgeBuilder::addLink(widget, src, target);
    }
    else {
        //the graph is a rule
        assert(PorgyTlpGraphStructure::isRuleGraph(g));
        PortGraphRuleDecorator dec(g);
        IntegerProperty *side = dec.getSideProperty();
        IntegerProperty *arity = g->getProperty<IntegerProperty>(PorgyConstants::ARITY);
        LayoutProperty* mLayout = g->getProperty<LayoutProperty>("viewLayout");
        Observable::holdObservers();
        if((dec.getSide(src)==PorgyConstants::SIDE_LEFT)&&(dec.getSide(target)==PorgyConstants::SIDE_RIGHT)) {
            setBends(vector<Coord>());
            PortGraphRule pgr(g);
            Bridge *b = pgr.getBridge();
            b->addPort(src, target); //add one node and 3 edges to the graph
            b->draw(g->getProperty<SizeProperty>("viewSize"), g->getProperty<ColorProperty>("viewColor"), mLayout, g->getProperty<ColorProperty>("viewBorderColor"));
            b->setPositions(mLayout);
        }
        else if((dec.getSide(src)==PorgyConstants::SIDE_LEFT)&&(dec.getSide(target)==PorgyConstants::SIDE_LEFT)) {
            edge e = g->addEdge(src, target);
            side->setEdgeValue(e, PorgyConstants::SIDE_LEFT);
            arity->setNodeValue(src, arity->getNodeValue(src)+1);
            arity->setNodeValue(target, arity->getNodeValue(target)+1);
            mLayout->setEdgeValue(e, bends());

        }
        else if((dec.getSide(src)==PorgyConstants::SIDE_RIGHT)&&(dec.getSide(target)==PorgyConstants::SIDE_RIGHT)) {
            edge e = g->addEdge(src, target);
            side->setEdgeValue(e, PorgyConstants::SIDE_RIGHT);
            arity->setNodeValue(src, arity->getNodeValue(src)+1);
            arity->setNodeValue(target, arity->getNodeValue(target)+1);
            mLayout->setEdgeValue(e, bends());
        }
        setBends(vector<Coord>());
        Observable::unholdObservers();
    }
}

bool PortGraphAddEdgeComponent::eventFilter(QObject *widget, QEvent *e) {    
    QMouseEvent *qMouseEv = (QMouseEvent *) e;
    GlMainWidget *glMainWidget = static_cast<GlMainWidget *>(widget);
    Graph * _graph = view()->graph();
    if ((e->type() == QEvent::MouseButtonPress)&&(qMouseEv->button() == Qt::LeftButton)) {
        SelectedEntity selectedEntity;
        glMainWidget->pickNodesEdges(qMouseEv->x(), qMouseEv->y(), selectedEntity, nullptr, true, false);
        node n(selectedEntity.getNode());
        if (n.isValid()) {
            if((PortBase::isPort(n, _graph))&&(!Bridge::isBridge(n, _graph))) {
                if(PorgyTlpGraphStructure::isRuleGraph(_graph)) {
                    PortGraphRuleDecorator dec(_graph);
                    if(!source().isValid())
                        return MouseEdgeBuilder::eventFilter(widget, e);
                    else if(!((dec.getSide(source())==PorgyConstants::SIDE_RIGHT)&&(dec.getSide(n)==PorgyConstants::SIDE_LEFT)))
                        return MouseEdgeBuilder::eventFilter(widget, e);
                    else
                        return false;
                }
                else
                    return MouseEdgeBuilder::eventFilter(widget, e);
            }
            else
                return false;
        }
    }
    else
        return MouseEdgeBuilder::eventFilter(widget, e);

    return MouseEdgeBuilder::eventFilter(widget, e);
}
