/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef NEWPROPERTY_H
#define NEWPROPERTY_H

#include <QDialog>
#include <QStyleOptionViewItem>
#include <QPainter>
#include <QModelIndex>
#include <QWidget>
#include <QItemDelegate>
#include <tulip/TlpQtTools.h>

class PorgyPropertyManager;

namespace Ui {
    class PorgyPropertyManager;
}

namespace tlp {
    class Graph;
}

class PorgyPropertyManagerDelegate : public QItemDelegate
{
    Q_OBJECT

public:
    PorgyPropertyManagerDelegate(PorgyPropertyManager *window, QObject *parent = 0): QItemDelegate(parent), parentManager(window) {}

    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    void setEditorData(QWidget *editor, const QModelIndex &index) const;
    void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;

private slots:
    void commitAndCloseEditor();
private:
    const static int propertyNameColumn = 0;
    const static int testToMatchColumn = 1;
    PorgyPropertyManager *parentManager;
};


class PorgyPropertyManager : public QDialog
{
    Q_OBJECT

public:
    ~PorgyPropertyManager();
    static void propertyManager(tlp::Graph* parentGraph, QWidget *parent);

    void setMatchingStatus(std::string propertyName, bool match);
    std::string propertyAt(int row);
    tlp::Graph* getGraph();

private:
    PorgyPropertyManager(QWidget *parent, tlp::Graph *graph);
    void fillPropertyTable();

    tlp::Graph *graph;
    Ui::PorgyPropertyManager *ui;

protected slots:
    void create();
    void remove();
    void done();
};

#endif
