/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "PorgyPropertyManager.h"
#include <portgraph/PorgyConstants.h>
#include <portgraph/PorgyTlpGraphStructure.h>

#include <tulip/BooleanProperty.h>

#include <QComboBox>
#include <QStringList>
#include <QString>
#include <iostream>

using namespace tlp;
using namespace std;

void PorgyPropertyManagerDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const {
    if (index.column() == testToMatchColumn) {
        QString text = index.model()->data(index, Qt::DisplayRole).toString();
        QStyleOptionViewItem myOption = option;
        myOption.displayAlignment = Qt::AlignRight | Qt::AlignVCenter;

        drawDisplay(painter, myOption, myOption.rect, text);
        drawFocus(painter, myOption, myOption.rect);
    } else {
        QItemDelegate::paint(painter, option, index);
    }
}

QWidget* PorgyPropertyManagerDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const {
    if (index.column() == testToMatchColumn) {
        QComboBox *toMatch = new QComboBox(parent);
        QStringList qsl;
        qsl << "false" << "true";
        toMatch->addItems(qsl);
        connect(toMatch, SIGNAL(released()), this, SLOT(commitAndCloseEditor()));
        return toMatch;
    } else {
        return QItemDelegate::createEditor(parent, option, index);
    }
}

void PorgyPropertyManagerDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const {
    if (index.column() == testToMatchColumn) {
        QComboBox *toMatch = qobject_cast<QComboBox *>(editor);
        if (QStringToTlpString(index.model()->data(index, Qt::DisplayRole).toString())=="true")
            toMatch->setCurrentIndex(1);
        else
            toMatch->setCurrentIndex(0);
    } else {
        QItemDelegate::setEditorData(editor, index);
    }
}

void PorgyPropertyManagerDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const {
    if (index.column() == testToMatchColumn) {
        QComboBox *toMatch = qobject_cast<QComboBox *>(editor);
        QStringList qsl;
        qsl << "false" << "true";
        toMatch->addItems(qsl);
        bool isMatch = (toMatch->currentIndex() == 1);
        if (isMatch) {
            model->setData(index, "true");
        } else {
            model->setData(index, "false");
        }
        std::string newMatchingProperty = parentManager->propertyAt(index.row());
        parentManager->setMatchingStatus(newMatchingProperty, isMatch);
        if (!parentManager->getGraph()->existLocalProperty(PorgyConstants::MATCHING_PROPERTIES_PREFIX + newMatchingProperty + PorgyConstants::MATCHING_PROPERTIES_SUFFIX)) {
            tlp::BooleanProperty *stateProperty = PorgyTlpGraphStructure::getRulesRoot(parentManager->getGraph())->getLocalProperty<BooleanProperty>(PorgyConstants::MATCHING_PROPERTIES_PREFIX + newMatchingProperty + PorgyConstants::MATCHING_PROPERTIES_SUFFIX);
            stateProperty->setAllNodeValue(true);
        }
    } else {
        QItemDelegate::setModelData(editor, model, index);
    }
}

void PorgyPropertyManagerDelegate::commitAndCloseEditor() {
    QComboBox *editor = qobject_cast<QComboBox *>(sender());
    emit commitData(editor);
    //emit closeEditor(editor);
}
