/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PORGYRULEGRAPHVIEW_H
#define PORGYRULEGRAPHVIEW_H

#include "abstractportgraphview.h"

#include <tulip/Coord.h>
#include <tulip/GlScene.h>

#include <portgraph/PorgyConstants.h>

class RuleViewConfigWidget;
class PortNodeQt;
class RuleAttributesWidget;
class RuleAlgorithmWidget;
class QGraphicsSceneDragDropEvent;
class QDragMoveEvent;

class PorgyRuleGraphView : public AbstractPortGraphView
{
    Q_OBJECT

    tlp::SelectedEntity selectedEntity;
    RuleAttributesWidget* _RuleAttributes;
    RuleAlgorithmWidget* _RuleAlgorithm;

public:
    PLUGININFORMATION(PorgyConstants::RULE_VIEW_NAME,"B. Pinaud, J. Dubois","04/01/2011","","1.0",PorgyConstants::CATEGORY_NAME)
    PorgyRuleGraphView(const tlp::PluginContext *);
    virtual ~PorgyRuleGraphView();

    bool eventFilter(QObject *object, QEvent *event);
    void graphChanged(tlp::Graph *);

protected:
     void setupWidget();
     void fillContextMenu(QMenu *menu, const QPointF &);
     PortGraph* buildPortGraph(tlp::Graph* graph) const;
     PortGraphDecorator* buildPortGraphDecorator(tlp::Graph* graph) const;
      void initGlParameters(tlp::GlGraphRenderingParameters& parameters)const;
      QList<QWidget*> configurationWidgets()const;
      bool checkInteractors();

private slots:    
    void pasteSelectionToLHS();
    void pasteSelectionToRHS();
    void selectRHS();
    void selectLHS();
    void invertSelection();

private:
    void selectSide(PorgyConstants::RuleSide);
    bool treatDragMoveEvent(QDragMoveEvent* evt);
    bool treatGraphicsDragMoveEvent(QGraphicsSceneDragDropEvent* evt);

signals:
    void copy_paste_other_side(const PorgyConstants::RuleSide side);
    void pastePortNodes(tlp::Coord &center,tlp::Graph* model,QList<PortNodeQt*> portNodes);
};

#endif // PORGYRULEGRAPHVIEW_H
