/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PORGYTRACEGRAPHVIEW_H
#define PORGYTRACEGRAPHVIEW_H

#include "AbstractPorgyGraphView.h"

#include <portgraph/PorgyConstants.h>

#include <tulip/GlScene.h>

namespace tlp{
    class Camera;
    class GlRect;
    class GlLayer;
}

class TraceViewConfigWidget;
class QGraphicsSceneDragDropEvent;
class QGraphicsRectItem;

class PorgyTraceGraphView : public AbstractPorgyGraphView
{
    Q_OBJECT

    QAction *centerViewAction;
    QAction *redrawAction;
    QAction* goAction;
    QAction* addRemoveAction;

    bool closing;

    TraceViewConfigWidget *configwidget;

    tlp::SelectedEntity entity;

    tlp::node dragNode;

    void updateEntities();
    /**
      * @brief Check if there is a meta node under the mouse cursor.
      * @return The node id if there is a valid meta node under the cursor.
      **/
    tlp::node pickMetaNodeUderCursor(const QPoint& cursorPosition) const;

    /**
     * @brief checkMetanodeUnderDragEvent Check if a metanode is under the pointer, update the dragNode var and the highlight entity.
     * @param event
     * @return
     */
    bool checkMetanodeUnderDragEvent(QGraphicsSceneDragDropEvent *event);

    bool checkDragEventValidity(QGraphicsSceneDragDropEvent *event);

    QGraphicsRectItem* _dragOverItem;

public:
    PLUGININFORMATION(PorgyConstants::TRACE_VIEW_NAME,"B. Pinaud, J. Dubois","04/01/2011","","1.0",PorgyConstants::CATEGORY_NAME)
    PorgyTraceGraphView(const tlp::PluginContext *);
    virtual ~PorgyTraceGraphView();

    bool eventFilter(QObject *object, QEvent *event);
    virtual QList<QWidget*> configurationWidgets() const;


signals:
    void applyRuleOnModel(tlp::Graph* model,tlp::Graph* rule);
    void applyStrategyOnModel(tlp::Graph* model,QString strategy);    
    void openModelView(tlp::Graph *g);

private slots:
    void redrawTraceGraph();
    void ViewMetaNodesLabels(bool state);
    void ViewNodesLabels(bool state);
    void ScaleNodesLabels(bool state);
    void EdgeSizeInterpolation(bool state);
    void invertSelection();
    void openModel();
    void applyColorMapping();
protected:
    void setupWidget();
    void initGlParameters(tlp::GlGraphRenderingParameters& parameters)const;
    void fillContextMenu(QMenu *contextMenu, const QPointF &position);
    void applySettings(){}
    void graphChanged(tlp::Graph *);
};

#endif // PORGYTRACEGRAPHVIEW_H
