/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef GLITEMNAVIGATIONINTERACTOR_H
#define GLITEMNAVIGATIONINTERACTOR_H

#include <QObject>

class GlItem;

/**
 * @brief The GlItemNavigationController class control the navigation for a GlItem. Use the MouseInteraction flag to control the navigation inside of the GlItem.
 */
class GlItemNavigationController : public QObject
{
public:
    enum MouseInteraction{Translation=0x1,Rotation=0x2,Zoom=0x4,All_Interaction=0x7};
    Q_DECLARE_FLAGS(MouseInteractions,MouseInteraction)
private:
    int x,y;
    bool inRotation;
    bool inZoom;
    MouseInteractions _interactions;
public:


    GlItemNavigationController(QObject* parent=nullptr);
    GlItemNavigationController(MouseInteractions interactions,QObject* parent=nullptr);

    MouseInteractions interactions()const{
        return _interactions;
    }

    void setInteractions(MouseInteractions interactions){
        _interactions = interactions;
    }

    void install(GlItem* item);
    bool eventFilter(QObject *, QEvent *);
};

#endif // GLITEMNAVIGATIONINTERACTOR_H
