/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "porgysmallmultiplesconfigurationwidget.h"

#include "ui_porgysmallmultiplesconfigurationwidget.h"

PorgySmallMultiplesConfigurationWidget::PorgySmallMultiplesConfigurationWidget(QWidget *parent):QWidget(parent),ui(new Ui::PorgySmallMultiplesConfigurationWidget) {
    ui->setupUi(this);
    connect(ui->displaySmallMultipleCheckBox,SIGNAL(toggled(bool)),this,SIGNAL(displaySmallMultiplesView(bool)));
    connect(ui->displayAnimationCheckBox,SIGNAL(toggled(bool)),this,SIGNAL(displayAnimationView(bool)));
    connect(ui->intermediateStatesButtonGroup,SIGNAL(buttonClicked(int)),this,SIGNAL(intermediateStatesChanged()));
    connect(ui->checkscalelabels, SIGNAL(toggled(bool)), this, SIGNAL(scalelabels(bool)));
    connect(ui->nodeLabelsGroupBox, SIGNAL(toggled(bool)), this, SIGNAL(nodeLabelsVisibilityChanged(bool)));
}

PorgySmallMultiplesConfigurationWidget::~PorgySmallMultiplesConfigurationWidget() {
    delete ui;
}

PorgyIntermediaryStateFlags PorgySmallMultiplesConfigurationWidget::intermediateStates()const{
    PorgyIntermediaryStateFlags flags;
    if(ui->viewPLHScheckBox->isChecked()){
        flags |= ViewLHSP;
    }
    if(ui->viewLHSCheckBox->isChecked()){
        flags |= ViewLHS;
    }
    if(ui->viewRHSCheckBox->isChecked()){
        flags |= ViewRHS;
    }
    return flags;
}

void PorgySmallMultiplesConfigurationWidget::setintermediateStates(const PorgyIntermediaryStateFlags &flags) {
    ui->viewRHSCheckBox->setChecked(flags.testFlag(ViewRHS));
    ui->viewLHSCheckBox->setChecked(flags.testFlag(ViewLHS));
    ui->viewPLHScheckBox->setChecked(flags.testFlag(ViewLHSP));
}

bool PorgySmallMultiplesConfigurationWidget::labelScaled()const{
    return ui->checkscalelabels->isChecked();
}

bool PorgySmallMultiplesConfigurationWidget::nodeLabelsVisible()const{
    return ui->nodeLabelsGroupBox->isChecked();
}

void PorgySmallMultiplesConfigurationWidget::setLabelScaled(const bool label) {
    ui->checkscalelabels->setChecked(label);
}

void PorgySmallMultiplesConfigurationWidget::setNodeLabelsVisible(const bool visible) {
    ui->nodeLabelsGroupBox->setChecked(visible);
}

void PorgySmallMultiplesConfigurationWidget::setShowAnimation(const bool show) {
    ui->displayAnimationCheckBox->setChecked(show);
}

void PorgySmallMultiplesConfigurationWidget::setShowSmallMultiples(const bool show) {
    ui->displaySmallMultipleCheckBox->setChecked(show);
}

bool PorgySmallMultiplesConfigurationWidget::showAnimation() const {
    return ui->displayAnimationCheckBox->isChecked();
}

bool PorgySmallMultiplesConfigurationWidget::showSmallMultiples() const {
    return ui->displaySmallMultipleCheckBox->isChecked();
}
