/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <portgraph/PorgyConstants.h>

#include "porgysmallmultiplestraceview.h"
#include "porgysmallmultiplestraceviewinteractor.h"
#include "glitem.h"

using namespace tlp;
using namespace std;

PorgySmallmultiplesTraceViewInteractor::PorgySmallmultiplesTraceViewInteractor(const PluginContext*):InteractorComposite(QIcon(":/tulip/gui/icons/i_navigation.png"),"Navigation")
{
}

void PorgySmallmultiplesTraceViewInteractor::construct(){
    push_front(new PorgyTraceAnimationViewNavigationComponent());
    push_front(new PorgyTraceAnimationViewSelectionComponent());
}

bool PorgySmallmultiplesTraceViewInteractor::isCompatible(const string &viewName) const {
    return (viewName == PorgyConstants::SMALL_MULTIPLE_VIEW_NAME);
}

PorgyTraceAnimationViewNavigationComponent::PorgyTraceAnimationViewNavigationComponent():_navigationController(GlItemNavigationController::MouseInteractions(GlItemNavigationController::Translation | GlItemNavigationController::Zoom)){

}

void PorgyTraceAnimationViewNavigationComponent::viewChanged(View *view){
    if (view==nullptr)
        return;
    PorgySmallMultiplesTraceView* v = dynamic_cast<PorgySmallMultiplesTraceView*>(view);
    v->smallMultipleItem()->setAcceptedMouseButtons(Qt::LeftButton);
    v->smallMultipleItem()->installEventFilter(&_navigationController);
}

void PorgyTraceAnimationViewSelectionComponent::viewChanged(tlp::View *view){
    if (view==nullptr)
        return;
    PorgySmallMultiplesTraceView* v = dynamic_cast<PorgySmallMultiplesTraceView*>(view);
    v->smallMultipleItem()->setAcceptedMouseButtons(Qt::LeftButton);
    v->smallMultipleItem()->installEventFilter(this);
}

bool PorgyTraceAnimationViewSelectionComponent::eventFilter(QObject *obj, QEvent *evt){
    GlItem* item = qobject_cast<GlItem*>(obj);
    if(item != nullptr){
        if(evt->type() == QEvent::GraphicsSceneMouseDoubleClick){
            QGraphicsSceneMouseEvent* me = static_cast<QGraphicsSceneMouseEvent*>(evt);

            tlp::SelectedEntity selectedEntity;
            if(item->pickNodesEdges(me->pos().x(),me->pos().y(),selectedEntity)){
                std::cout<<__PRETTY_FUNCTION__<<" Double click "<<selectedEntity.getComplexEntityId()<<std::endl;
            }else{
                std::cout<<__PRETTY_FUNCTION__<<" Double click No items"<<std::endl;
            }
            return true;
        }
    }
    return false;
}

PLUGIN(PorgySmallmultiplesTraceViewInteractor)
