import QtQuick 1.1
import Tulip 1.0
Item {

    signal seekBackward
    signal skipBackward
    signal play
    signal pause
    signal stop
    signal skipForward
    signal seekForward

    property alias remoteMode: remote.mode
    property alias stateLabelOpacity: state_name.opacity
    property alias stateLabel: state_name.text
    property alias rulePreviewLabel: rule_preview_label.text

    property alias rulePreviewVisible: rule_preview.visible
    property alias animationVisible: animationGroup.visible
    property alias smallMultipleVisible: gl.visible    
    property alias rulePreviewGeometry: rule_preview.state

    id: p;
    width: 200
    height: 200

    Column {
        id: row1
        objectName: "column"
        anchors.fill: parent

        GlItem{
            id: gl
            objectName: "SmallMultipleItem"
            anchors.right: parent.right
            anchors.rightMargin: 0
            anchors.left: parent.left
            anchors.leftMargin: 0
            height: parent.height/2
            onVisibleChanged: {
                if(visible){
                    animationGroup.height = parent.height/2
                    height = parent.height/2
                }else{
                    animationGroup.height = parent.height
                }
            }
        }
        Rectangle {
            id: animationGroup
            objectName: "AnimationItem"
            height: parent.height/2
            anchors.topMargin: 0
            anchors.right: parent.right
            anchors.rightMargin: 0
            anchors.left: parent.left
            anchors.leftMargin: 0
            onVisibleChanged: {
                if(visible){
                    gl.height = parent.height/2
                    height = parent.height/2
                }else{
                    gl.height = parent.height
                }
            }

            GlItem{
                id: animationGlItem
                anchors.right: parent.right
                anchors.rightMargin: 0
                anchors.left: parent.left
                anchors.leftMargin: 0
                objectName: "GlItem"
                height: animationGroup.height - remote.height
                anchors.top: parent.top
                anchors.topMargin: 0



                Rectangle {
                    id : rule_preview
                    width: 100
                    height: 120
                    border.color: "#000000"
                    anchors.bottom: animationGlItem.bottom
                    anchors.right: animationGlItem.right
                    visible: false


                    state: "Small"

                    states:[
                        State {
                            name: "Small"
                            PropertyChanges {
                                target: rule_preview
                                width: 100
                                height: 120
                            }
                        },
                        State {
                            name: "Medium"
                            PropertyChanges {
                                target: rule_preview
                                width: 150
                                height: 170
                            }
                        },State {
                            name: "Large"
                            PropertyChanges {
                                target: rule_preview
                                width: 200
                                height: 220
                            }
                        }
                    ]

                    Row{
                        id: row2
                        GlItem {
                            id: gl1
                            height: row2.height - rule_preview_label.height
                            objectName: "RulePreview"
                        }
                        TextEdit {
                            id: rule_preview_label
                            height: 20
                            text: qsTr("text edit")
                            horizontalAlignment: TextEdit.AlignHCenter
                            verticalAlignment: TextEdit.AlignVCenter                            
                            font.pixelSize: 12
                        }
                    }
                }

                Text {
                    id: state_name
                    anchors.left: parent.left
                    anchors.bottom: parent.bottom
                    text: qsTr("text")
                    font.pixelSize: 12
                    objectName: "StateName"
                }

            }
            Remote {
                id : remote
                objectName: "Remote"
                anchors.right: parent.right
                anchors.rightMargin: 0
                anchors.left: parent.left
                anchors.leftMargin: 0
                height: 20
                anchors.top: animationGlItem.bottom
                anchors.topMargin: 0

                onPlay: {p.play();}
                onPause: {p.pause();}
                onStop: {p.stop();}
                onSeekBackward: {p.seekBackward();}
                onSkipBackward: {p.skipBackward();}
                onSeekForward: {p.seekForward();}
                onSkipForward: {p.skipForward();}
            }
        }


    }


}


