/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "smallmultiplesconfigurationwidget.h"

#include "ui_smallmultiplesconfigurationwidget.h"

SmallMultiplesConfigurationWidget::SmallMultiplesConfigurationWidget(QWidget *parent):QWidget(parent), ui(new Ui::SmallMultiplesConfigurationWidget) {
    ui->setupUi(this);
    connect(ui->smallMultipleLayoutButtonGroup,SIGNAL(buttonClicked( int)),this,SIGNAL(smallMultiplesLayoutUpdated()));
    connect(ui->lineNumberSpinBox,SIGNAL(valueChanged(int)),this,SIGNAL(smallMultiplesLayoutUpdated()));    
    connect(ui->scaleTitlesCheckBox, SIGNAL(toggled(bool)), this, SIGNAL(scaleTitles(bool)));
    connect(ui->space, SIGNAL(valueChanged(double)), this, SIGNAL(spaceChanged(double)));
}

SmallMultiplesConfigurationWidget::~SmallMultiplesConfigurationWidget() {
    delete ui;
}

bool SmallMultiplesConfigurationWidget::titlesScaled()const {
    return ui->scaleTitlesCheckBox->isChecked();
}

void SmallMultiplesConfigurationWidget::setTitlesScaled(bool scaled) {
    ui->scaleTitlesCheckBox->setChecked(scaled);
}

double SmallMultiplesConfigurationWidget::spacing()const{
    return ui->space->value();
}

void SmallMultiplesConfigurationWidget::setSpacing(const double &space) {
    ui->space->setValue(space);
}

SmallMultiplesLayout SmallMultiplesConfigurationWidget::smallMultiplesLayout() const{
    if(ui->automaticLayoutRadioButton->isChecked()){
        return Automatic;
    }else if(ui->forceLineNumberRadioButton->isChecked()){
        return ForceLineNumber;
    }else{
        //Never called.
        return Automatic;
    }
}

void SmallMultiplesConfigurationWidget::setLayout(const SmallMultiplesLayout layout) {
    switch(layout) {
    case Automatic:
        ui->automaticLayoutRadioButton->setChecked(true);
        ui->forceLineNumberRadioButton->setChecked(false);
        break;
    case ForceLineNumber:
        ui->forceLineNumberRadioButton->setChecked(true);
        ui->automaticLayoutRadioButton->setChecked(false);
        break;
    }
}

unsigned int SmallMultiplesConfigurationWidget::lineNumber()const{
    return ui->lineNumberSpinBox->value();
}

void SmallMultiplesConfigurationWidget::setLineNumber(const unsigned num) {
    ui->lineNumberSpinBox->setValue(num);
}
