/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <porgy/ModelElementValueModel.h>
#include <porgy/PropertyValueToQVariantManager.h>
#include <porgy/PropertyValueToQVariantConverter.h>
#include <porgy/PortNodeQt.h>

#include <portgraph/PorgyTlpGraphStructure.h>
#include <portgraph/PorgyConstants.h>

#include <tulip/StringProperty.h>
#include <tulip/BooleanProperty.h>
#include <tulip/Graph.h>

#include <unordered_set>

using namespace tlp;
using namespace std;

ModelElementValueModel::ModelElementValueModel(Graph* graph, ElementType elementType, unsigned int elementId, QObject* parent ):GraphElementValueModel(graph,elementType,elementId,unordered_set<string>(),parent) {
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::PORTNODE_TYPE,new PortNodeTypeNodePropertyConverter());
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::TAG,new UneditableNodeValueToQVariantConverter<IntegerProperty, int>());
    propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::TAG,new UneditableEdgeValueToQVariantConverter<IntegerProperty, int>());
  //  propertyDataConverterManager()->overloadNodePropertyConverter("viewSelection", new PortNodeBooleanPropertyConverter(graph));
  //  propertyDataConverterManager()->overloadNodePropertyConverter("viewSelection", new PortNodeBooleanPropertyConverter(graph));
    PropertyInterface* property;
    //do not edit property of a graph already used in a rewriting operation.
    //this code does not work well on string property.
//    if(PorgyTlpGraphStructure::isGraphUsed(graph)) {
//        forEach(property,graph->getObjectProperties()) {
//            if((property->getTypename() == BooleanProperty::propertyTypename)&&(property->getName()!="viewSelection")){
//                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableNodeValueToQVariantConverter<BooleanProperty,bool>());
//                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableEdgeValueToQVariantConverter<BooleanProperty,bool>());
//            }
////            if(property->getTypename() == IntegerProperty::propertyTypename){
////                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableNodeValueToQVariantConverter<IntegerProperty,int>());
////                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableEdgeValueToQVariantConverter<IntegerProperty,int>());
////            }
//            if(property->getTypename() == DoubleProperty::propertyTypename){
//                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableNodeValueToQVariantConverter<DoubleProperty,double>());
//                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableEdgeValueToQVariantConverter<DoubleProperty,double>());
//            }
//            if(property->getTypename() == StringProperty::propertyTypename){
//                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableNodeValueToQVariantConverter<StringProperty,string>());
//                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableEdgeValueToQVariantConverter<StringProperty,string>());
//            }
//            if(property->getTypename() == ColorProperty::propertyTypename){
//                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableNodeValueToQVariantConverter<ColorProperty,Color>());
//                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableEdgeValueToQVariantConverter<ColorProperty,Color>());
//            }
//        }
//    }
//    else {
        forEach(property,graph->getObjectProperties()) {
            if(property->getTypename() == BooleanProperty::propertyTypename){
                propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new PortNodeBooleanPropertyConverter(graph));
                //     propertyDataConverterManager()->overloadNodePropertyConverter(property->getName(), new UneditableNodeValueToQVariantConverter<BooleanProperty, bool>());
            }
        }
//    }

    initPropertiesList(buildPropertiesList());
}

unordered_set<string> ModelElementValueModel::buildPropertiesList(){
    unordered_set<string> propertiesNames;
    PorgyTlpGraphStructure::getMatchingPropertyNames(graph(), propertiesNames);
    propertiesNames.insert(PorgyConstants::TAG);
    propertiesNames.insert("viewBorderWidth");
    propertiesNames.insert("viewColor");
    propertiesNames.insert("viewBorderColor");
    propertiesNames.insert("viewSelection");

    //Search for boolean properties : viewSelection and P properties.
    PropertyInterface* property;
    forEach(property,graph()->getObjectProperties())
    {
        if(property->getTypename() == BooleanProperty::propertyTypename){
            propertiesNames.insert(property->getName());
        }
    }
    switch(elementType()) {
        case NODE:
        {
            propertiesNames.insert("viewLabel");
            propertiesNames.insert("viewShape");
            propertiesNames.insert(PorgyConstants::PORTNODE_TYPE);
        break;
        }
        case EDGE:
        {
            propertiesNames.insert("viewSrcAnchorSize");
            propertiesNames.insert("viewSrcAnchorShape");
            propertiesNames.insert("viewTgtAnchorSize");
            propertiesNames.insert("viewTgtAnchorShape");
            break;
        }
    }

    return propertiesNames;
}
