/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <porgy/RuleElementValueModel.h>
#include <porgy/PropertyValueToQVariantManager.h>
#include <porgy/PropertyValueToQVariantConverter.h>
#include <porgy/PortNodeQt.h>

#include <portgraph/PorgyTlpGraphStructure.h>

#include <tulip/Graph.h>
#include <tulip/StringProperty.h>
#include <tulip/BooleanProperty.h>
#include <tulip/IntegerProperty.h>

using namespace tlp;
using namespace std;

/**
 * @brief The TransformationTypePropertyConverter class converter for PorgyConstants::RuleSide enum type.
 */
class RuleSideNodePropertyConverter : public AbstractEnumNodePropertyEditor{
public:
    QString label(const unsigned int& value)const{
         return PorgyConstants::RuleSideToString.at(static_cast<PorgyConstants::RuleSide>(value));
    }
};

class RuleSideEdgePropertyConverter : public AbstractEnumEdgePropertyEditor{
public:
    QString label(const unsigned int& value)const{
        return PorgyConstants::RuleSideToString.at(static_cast<PorgyConstants::RuleSide>(value));
    }
};

RuleElementValueModel::RuleElementValueModel(Graph* graph, ElementType elementType, unsigned int elementId, QObject* parent ):GraphElementValueModel(graph,elementType,elementId,unordered_set<string>(),parent)
{
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::RULE_SIDE,new RuleSideNodePropertyConverter());
    propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::RULE_SIDE,new RuleSideEdgePropertyConverter());
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::PORTNODE_TYPE,new PortNodeTypeNodePropertyConverter());
    propertyDataConverterManager()->overloadNodePropertyConverter("viewSelection", new PortNodeBooleanPropertyConverter(graph));
    if(PorgyTlpGraphStructure::isRuleUsed(graph)) {
        //Avoid modification to all properties
        propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::M, new UneditableNodeValueToQVariantConverter<tlp::BooleanProperty,bool>());
        propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::N, new UneditableNodeValueToQVariantConverter<tlp::BooleanProperty,bool>());
        propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::ARITY, new UneditableNodeValueToQVariantConverter<tlp::IntegerProperty,unsigned int>());
        propertyDataConverterManager()->overloadNodePropertyConverter("viewColor", new UneditableNodeValueToQVariantConverter<tlp::ColorProperty,Color>());
        propertyDataConverterManager()->overloadNodePropertyConverter("viewBorderColor", new UneditableNodeValueToQVariantConverter<tlp::ColorProperty,Color>());
        propertyDataConverterManager()->overloadNodePropertyConverter("viewBorderWidth", new UneditableNodeValueToQVariantConverter<tlp::DoubleProperty,double>());
        propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::M, new UneditableEdgeValueToQVariantConverter<tlp::BooleanProperty,bool>());
        propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::N, new UneditableEdgeValueToQVariantConverter<tlp::BooleanProperty,bool>());
        propertyDataConverterManager()->overloadEdgePropertyConverter("viewLabel", new UneditableEdgeValueToQVariantConverter<tlp::StringProperty,string>());
        propertyDataConverterManager()->overloadNodePropertyConverter("viewLabel", new UneditableEdgeValueToQVariantConverter<tlp::StringProperty,string>());
        propertyDataConverterManager()->overloadEdgePropertyConverter("viewColor", new UneditableEdgeValueToQVariantConverter<tlp::ColorProperty,Color>());
        propertyDataConverterManager()->overloadEdgePropertyConverter("viewBorderColor", new UneditableEdgeValueToQVariantConverter<tlp::ColorProperty,Color>());
        propertyDataConverterManager()->overloadEdgePropertyConverter("viewBorderWidth", new UneditableEdgeValueToQVariantConverter<tlp::DoubleProperty,double>());

        //TODO
//#warning manage the property checked
    }
    else {
        //Avoid modification on RULE_SIDE property
        propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::M, new PortNodeBooleanPropertyConverter(graph));
        propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::N, new PortNodeBooleanPropertyConverter(graph));
        propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::M, new PortNodeBooleanPropertyConverter(graph));
        propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::N, new PortNodeBooleanPropertyConverter(graph));
        propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::ARITY, new NodeValueToQVariantConverter<tlp::IntegerProperty,unsigned int>());
    }
    initPropertiesList(buildPropertiesList());

}

unordered_set<string> RuleElementValueModel::buildPropertiesList(){
    unordered_set<string> propertiesNames;
    PorgyTlpGraphStructure::getMatchingPropertyNamesExtended(graph(), propertiesNames);
    propertiesNames.insert(PorgyConstants::M);
    propertiesNames.insert(PorgyConstants::N);
    propertiesNames.insert(PorgyConstants::RULE_SIDE);
    propertiesNames.insert("viewColor");
    propertiesNames.insert("viewBorderColor");
    propertiesNames.insert("viewBorderWidth");
    propertiesNames.insert("viewSelection");
    switch(elementType()) {
    case NODE:
        propertiesNames.insert("viewLabel");
        propertiesNames.insert(PorgyConstants::PORTNODE_TYPE);
        propertiesNames.insert(PorgyConstants::ARITY);
        break;
    case EDGE:
        break;
    }

    return propertiesNames;
}
