/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef GRAPHELEMENTVALUEMODEL_H
#define GRAPHELEMENTVALUEMODEL_H
#include <QAbstractTableModel>
#include <QVector>

#include <tulip/Graph.h>

#include <unordered_set>

class PropertyValueToQVariantManager;
/**
 * @brief The GraphElementValueModel class Displays properties for given element according to it's type and id.
 * To filter the displayed properties give the name of the properties to visualize
 */
class GraphElementValueModel : public QAbstractTableModel
{

    PropertyValueToQVariantManager* _propertyDataConverter;
    tlp::Graph* _graph;
    tlp::ElementType _displayType;
    unsigned int _eltId;
    QVector<tlp::PropertyInterface*> _propertyTable;
public:
    /**
     * @brief GraphElementValueModel Display only properties with the name given in the propertiesToDisplay parameter.
     * @param graph
     * @param eltType
     * @param eltId
     * @param propertiesToDisplay
     * @param parent
     */
    GraphElementValueModel(tlp::Graph* graph, tlp::ElementType eltType, unsigned int eltId,const std::unordered_set<std::string>& propertiesToDisplay,QObject* parent = 0);
    /**
     * @brief GraphElementValueModel Displays all properties available in the graph.
     * @param graph
     * @param eltType
     * @param eltId
     * @param parent
     */
    GraphElementValueModel(tlp::Graph* graph, tlp::ElementType eltType, unsigned int eltId,QObject* parent = 0);


    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
    int columnCount(const QModelIndex& parent = QModelIndex()) const;
    int rowCount(const QModelIndex& parent = QModelIndex()) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
    bool setData( const QModelIndex & index, const QVariant & value, int role = Qt::EditRole );
    Qt::ItemFlags flags ( const QModelIndex & index ) const;
    void sort ( int column, Qt::SortOrder order = Qt::AscendingOrder );


    tlp::ElementType elementType()const{
        return _displayType;
    }

    unsigned int elementId()const{
        return _eltId;
    }

    tlp::Graph* graph()const{
        return _graph;
    }

    PropertyValueToQVariantManager* propertyDataConverterManager(){
        return _propertyDataConverter;
    }

protected:
    void initPropertiesList(const std::unordered_set<std::string>& propertiesToDisplay);
//    void addWithoutDuplicate(std::vector<std::string>& propertiesToDisplay, const std::string value);
};

#endif // GRAPHELEMENTVALUEMODEL_H
