/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <tulip/IntegerProperty.h>
#include <tulip/StringCollection.h>
#include <tulip/BooleanProperty.h>

#include <portgraph/Port.h>
#include <portgraph/PorgyConstants.h>
#include <portgraph/PortNode.h>
#include <portgraph/PortGraph.h>
#include <portgraph/PorgyTlpGraphStructure.h>

using namespace std;
using namespace tlp;

//#define NDEBUG

Port::Port(const tlp::node n, const PortNode *pp):PortBase(n,pp){
}

const PortNode* Port::getOneConnectedPortNode() const{
    return connectedPorts.empty()?nullptr:connectedPorts[0]->getParentPortNode();
}

bool Port::check(const PortPtr rule_port, const PortPtr modele_port, bool exact) {
#ifndef NDEBUG
    cerr << __PRETTY_FUNCTION__ << ": checking model port " << modele_port->getNode() << " and rule port " << rule_port->getNode() << endl;
#endif

    //if the rule_port has a degree of 1 inside the rule, then it is not connected to RHS but only to its center => exact matching required
    //rule_port is not coming from a PortGraphRule but from a PortGraphModel (ok, it's an ugly hack)
    if(rule_port->getParentPortNode()->getParentPortGraph().getGraph()->getSuperGraph()->deg(rule_port->getNode())==1)
        exact=true;

    if( (exact && (rule_port->deg() == modele_port->deg())) || (!exact && (rule_port->getArity() <= modele_port->deg()))) {
    //checking the rest of the matching properties
        vector<string> matchingPropertiesList;
        if (modele_port->getParentPortNode()->getParentPortGraph().getGraph()->getRoot()->attributeExist(PorgyConstants::MATCHING_PROPERTIES)) {
            modele_port->getParentPortNode()->getParentPortGraph().getGraph()->getRoot()->getAttribute<vector<string> >(PorgyConstants::MATCHING_PROPERTIES, matchingPropertiesList);
            for(auto& propertyName: matchingPropertiesList) {
                PropertyInterface* currentModelProperty = modele_port->getParentPortNode()->getParentPortGraph().getGraph()->getProperty(propertyName);
                PropertyInterface* currentRuleProperty = rule_port->getParentPortNode()->getParentPortGraph().getGraph()->getProperty(propertyName);
                BooleanProperty* testCurrentProperty = rule_port->getParentPortNode()->getParentPortGraph().getGraph()->getProperty<BooleanProperty>(PorgyConstants::MATCHING_PROPERTIES_PREFIX + propertyName + PorgyConstants::MATCHING_PROPERTIES_SUFFIX);
                if(testCurrentProperty->getNodeValue(rule_port->getNode())) {
                    if(currentRuleProperty->getNodeStringValue(rule_port->getNode()) != currentModelProperty->getNodeStringValue(modele_port->getNode())) {
#ifndef NDEBUG
                        cerr << __PRETTY_FUNCTION__ << ": port problem: property " << propertyName << endl;
                        cerr << __PRETTY_FUNCTION__ << ": model port: " << modele_port->getNode() <<" and rule port " << rule_port->getNode() << " not compatible" << endl;
#endif
                        return false;
                    }
                }
            }
        } /*else {
#ifndef NDEBUG
            cerr << __PRETTY_FUNCTION__ << ": no known matching properties, using default value." << endl;
#endif
            PropertyInterface *default_p_rule = rule_port->getParentPortNode()->getParentPortGraph()->getGraph()->getProperty(PorgyConstants::MATCHING_PROPERTY_DEFAULT);
            PropertyInterface *default_p_model = modele_port->getParentPortNode()->getParentPortGraph()->getGraph()->getProperty(PorgyConstants::MATCHING_PROPERTY_DEFAULT);
            if(default_p_rule->getNodeStringValue(rule_port->getNode()) != default_p_model->getNodeStringValue(modele_port->getNode()))
                return false;
        }*/

        return true;
    }
#ifndef NDEBUG
    else
        cerr << __PRETTY_FUNCTION__ << ": Problem with port degrees (model): " << modele_port->deg()  << " and (rule)" << rule_port->deg() << endl;
#endif

    return false;
}

unsigned int Port::getArity() const {
#ifndef NDEBUG
    if(!PorgyTlpGraphStructure::isRuleGraph(getParentPortNode()->getParentPortGraph().getGraph()))
        std::cerr << __PRETTY_FUNCTION__ << " WARNING: graph is not a rule" << std::endl;
#endif
    tlp::IntegerProperty *arity = getParentPortNode()->getParentPortGraph().getGraph()->getProperty<tlp::IntegerProperty>(PorgyConstants::ARITY);
    return static_cast<unsigned int>(arity->getNodeValue(getNode()));
}

unsigned int Port::deg() const {
    assert(getParentPortNode()->getParentPortGraph().getGraph()->isElement(getNode()));
    return getParentPortNode()->getParentPortGraph().getGraph()->deg(getNode())-1;
}
