/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _PORT_H
#define _PORT_H

#include <portgraph/PortBase.h>
#include <portgraph/PorgyTlpGraphStructure.h>

#include <memory>

class Port : public PortBase {

public :

    Port(const tlp::node, const PortNode *);

    //implicit conversion from Port to tlp::node
    operator tlp::node() const {
        return this->getNode();
    }

    //shared_ptr used because a port may be used in many portnodes throw the connectedPorts member vector
    typedef std::shared_ptr<Port> PortPtr;

    inline tlp::Iterator<Port::PortPtr> *getConnectedPorts() const {
        return new CopyStlIterator<Port::PortPtr, std::vector<PortPtr> >(connectedPorts);
    }
    inline std::vector <PortPtr>::const_iterator connectedPortsBegin() const {return connectedPorts.begin();}

    const PortNode *getOneConnectedPortNode() const;
    unsigned int getArity() const;

    /**
     * @brief check if two ports are equivalent
     * @return
     */
    static bool check(const PortPtr rule_port, const PortPtr modele_port, bool exact);
    unsigned int deg() const;
    inline void connectPortTo(const PortPtr p) {connectedPorts.push_back(p);}

private:
    std::vector<PortPtr>connectedPorts;		//liste des Ports distants connectés à ce port

};

#endif
