/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _PORTGRAPH_H
#define _PORTGRAPH_H

#include <tulip/GraphDecorator.h>

#include <portgraph/Port.h>
#include <portgraph/PortNode.h>

namespace tlp {
class Graph;
class SizeProperty;
class LayoutProperty;
class PropertyInterface;
}

class Bridge;

class PortGraphDecorator: public tlp::GraphDecorator {
public:
    PortGraphDecorator(tlp::Graph *g);
    virtual tlp::BooleanProperty *getSelectionProperty()=0;
    /**
     * @brief redraw Redraw the model using the given plugin. If errors occurs, they are store in the errMsg variable.
     * The plugin must be in the redrawPlugins list.
     * @param errMsg The error message if the redraw process fail.
     * @return True if the redraw was done correctly.
     */
    virtual bool redraw(const std::string& plugin, tlp::DataSet& parameters ,std::string &errMsg,tlp::PluginProgress* progress=nullptr)=0;
    /**
     * @brief layoutPluginsList returns the list of the plugins capable to redraw a port graph. Call the redraw function with the selected plugins and it's parameters.
     * @return
     */
    virtual std::list<std::string> layoutPluginsList()const=0;
    //overloaded methods
    unsigned numberOfNodes() const;

    static bool isCenter(tlp::Graph *g, const tlp::node n);
};

class PortGraph {

    friend class Bridge;
    friend class PortNode;

public:

    typedef std::deque<PortNode *> dequeportnode;

    PortGraph (tlp::Graph *);
    PortGraph (const std::set<tlp::node> v, tlp::Graph *g);
    virtual ~PortGraph();

    void addPortContainer(const tlp::node &, Port::PortPtr);
    const Port::PortPtr getPortContainer(const tlp::node &) const;

    /**
     * @brief getPortNodes
     * @param ban
     * @return an iterator on all portnodes of the graph
     */
    virtual tlp::Iterator<PortNode *> *getPortNodes(tlp::BooleanProperty *ban=nullptr, bool random_order=false) const;
    inline tlp::Graph *getGraph() const {return _graph;}

    void drawSameSizes(tlp::SizeProperty *nodeSize, tlp::LayoutProperty *layout) const;

    static bool isPortGraphEdge(const tlp::edge e, tlp::Graph *g);

    virtual tlp::Iterator<tlp::edge> *getEdges(tlp::BooleanProperty *ban=nullptr)=0;
    /**
     * @brief getInOutPortNode : returns an iterator over all neighbours of the given portnode
     * @param n
     * @param ban
     * @return
     */
    tlp::Iterator<PortNode *> *getInOutPortNodes(PortNode* pn, tlp::BooleanProperty *ban=nullptr) const;

    //retourne le premier portnode
    PortNode *getOnePortNode() const;

    const std::pair<PortNode *, PortNode *> ends(const tlp::edge e) const;
    PortNode* operator[](const tlp::node &) const;
    PortNode*& operator[](const tlp::node &);

protected:

    tlp::Graph *_graph;

    inline void setPortNode(dequeportnode v) {portnode = v;}

   private:

    dequeportnode portnode;

    std::map<tlp::node, PortNode *> container;	//relation entre node et PortNode *
    std::map<tlp::node, Port::PortPtr> container_port;   //relation entre node et shared_ptr<Port>
};

#endif
