/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _PORTGRAPHRULE_H
#define _PORTGRAPHRULE_H

#include <portgraph/PortGraph.h>
#include <portgraph/PorgyConstants.h>

#include <tulip/StringCollection.h>

class PortNode;
class Bridge;

class PortGraphRule:public PortGraph {

    Bridge *bridge;

public:

    //iterators
    tlp::Iterator<PortNode *> *getMember(const PorgyConstants::RuleSide);
    /**
     * @brief getPortNodes
     * @return An iterator on the portnodes of the left- and right-hand side
     */
    virtual tlp::Iterator<PortNode *> *getPortNodes(tlp::BooleanProperty *ban=nullptr) const;
    tlp::StringCollection getLayoutAlgorithm();
    tlp::Iterator<tlp::edge> *getMemberEdges(const PorgyConstants::RuleSide);
    /**
     * @brief getAllEdges
     * @return Returns edges of the left and right members
     */

    tlp::Iterator<tlp::edge> *getEdges(tlp::BooleanProperty *ban=nullptr);

    inline Bridge * getBridge() const {return bridge;}

    static std::pair<tlp::Graph *,tlp::Graph *> extractLHSRHS(tlp::Graph *g, tlp::PluginProgress *pp, std::string &errMsg);
    static void cleanLHSRHS(tlp::Graph *g);

    PortGraphRule (tlp::Graph *);
    virtual ~PortGraphRule();
    tlp::node findRightEquiv(const tlp::node &n) const;
};

class PortGraphRuleDecorator: public PortGraphDecorator {
public:
    PortGraphRuleDecorator(tlp::Graph *g=nullptr);
    tlp::BooleanProperty *getSelectionProperty();
    virtual bool redraw(const std::string& plugin, tlp::DataSet& parameters ,std::string &errMsg,tlp::PluginProgress* progress=nullptr);
    std::list<std::string> layoutPluginsList()const;

    PorgyConstants::RuleSide getSide(const tlp::node &n);
    PorgyConstants::RuleSide getSide(const tlp::edge &e);

    /**
     * @brief getSideProperty
     * @param g
     * @return
     */
    inline tlp::IntegerProperty* getSideProperty() {
        return graph_component->getProperty<tlp::IntegerProperty>(PorgyConstants::RULE_SIDE);
    }
    //oveloaded methods
};

#endif
