/**
 *
 * This file is part of Porgy (http://tulip.labri.fr/TulipDrupal/?q=porgy)
 *
 * from LaBRI, University of Bordeaux, Inria and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _PORTNODE_H
#define _PORTNODE_H

#include <portgraph/PortNodeBase.h>
#include <portgraph/Port.h>

namespace tlp{
class Graph;
class SizeProperty;
class ColorProperty;
class LayoutProperty;
}

class PortNode:public PortNodeBase {

    std::vector<Port::PortPtr> ports;

    void setOptimalSize(const int, tlp::node, tlp::SizeProperty *nodeSize) const;
    void mySortPorts(std::vector<Port::PortPtr> &, const bool, const bool, tlp::LayoutProperty *layout ) const;
    void placePorts(tlp::LayoutProperty *layout, std::vector<Port::PortPtr> & , std::vector<Port::PortPtr> & ,std::vector<Port::PortPtr> & , std::vector<Port::PortPtr> & , const double , const double ) const;
    std::pair<unsigned,unsigned> drawPosition_aux(const bool, tlp::LayoutProperty *layout, const bool specifyWidths=false, const int xWidth=0, const int yWidth=0) const;
    // If draw && specifyWidhts, it draws the portnode with xWidth and yWidth.
    // If !draw, it calculates the desired withs iof the portnode
    void placePorts(tlp::LayoutProperty *layout, std::vector<Port::PortPtr> & , std::vector<Port::PortPtr> & , std::vector<Port::PortPtr> & , std::vector<Port::PortPtr> &) const;

public :

    PortNode(PortGraph &, const tlp::node);
    tlp::Iterator<const PortNode *> *getInOutConnectedPortNodes() const;

    PortNodeType getType();

    //renvoie un iterateur sur l'ensemble des ports
    inline tlp::Iterator<Port::PortPtr> *getPorts() const {
        return new CopyStlIterator<Port::PortPtr, std::vector<Port::PortPtr> >(ports);
    }

    inline unsigned nbPorts() const {return ports.size();}

    void resizeCenter(tlp::SizeProperty *nodeSize, tlp::LayoutProperty *layout) const;
    void resizeCenter(const double, const double, tlp::SizeProperty *nodeSize) const;
    virtual void setPositions(tlp::LayoutProperty *layout) const ;
    void setPositions(const int, const int, tlp::LayoutProperty *layout) const;
    std::pair<unsigned, unsigned> getOptimalSize(tlp::LayoutProperty *layout) const; // gives the optimal number of ports in x and y of the portnode

    unsigned deg() const;
    unsigned indeg() const;
    unsigned outdeg() const;

    void draw(tlp::SizeProperty *nodeSize, tlp::ColorProperty *viewColor, tlp::LayoutProperty *layout, tlp::ColorProperty *bordercolor) const;

    static bool check(const PortNode * rule_portnode, const PortNode* modele_portnode, const bool exact);
//    void select(const bool state) const;
    void select(const bool state,tlp::BooleanProperty* resultProperty) const;
};

#endif
